 /*
 *  Copyright (C) 2000 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include "mozilla_i18n.h"
#include "mozilla.h"
#include "galeon-wrapper.h"

#include <gtkmozembed_internal.h>
/* FIXME !!! */
#include <gnome.h>
#include <time.h>

#include "nsString.h"
#include "nsIPref.h"
#include "nsICharsetConverterManager.h"
#include "nsICharsetConverterManager2.h"
#include "nsIUnicodeEncoder.h"
#include "nsIPrintOptions.h"
#include "nsGfxCIID.h"
#include "nsICacheService.h"
#include "nsIPassword.h"
#include "nsIPasswordManager.h"
#include "nsIDOMWindow.h"
#include "nsIEmbeddingSiteWindow.h"
#include "nsIWindowWatcher.h"
#include "nsIWebBrowserChrome.h"

#include "PromptService.h"
#include "ContentHandler.h"
#include "ContentOverrideHandler.h"
#include "FilePicker.h"

static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);

/* local function prototypes */
static char  *convert_ns_string_to_c_string (const nsString & ns_string);

extern "C" gboolean
mozilla_load_prefs (void)
{
#if MOZILLA_VERSION > VERSION2(0,9)
	nsCOMPtr<nsIPrefService> prefService = 
				do_GetService (NS_PREFSERVICE_CONTRACTID);
	g_return_val_if_fail (prefService != nsnull, FALSE);

	nsresult rv = prefService->ReadUserPrefs (nsnull);
	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
#else
	return FALSE;
#endif
}

extern "C" gboolean
mozilla_save_prefs (void)
{
#if MOZILLA_VERSION > VERSION2(0,9)
	nsCOMPtr<nsIPrefService> prefService = 
				do_GetService (NS_PREFSERVICE_CONTRACTID);
	g_return_val_if_fail (prefService != nsnull, FALSE);

	nsresult rv = prefService->SavePrefFile (nsnull);
	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
#else
	return FALSE;
#endif
}

/**
 * mozilla_preference_set: set a string mozilla preference
 */
extern "C" gboolean
mozilla_preference_set(const char *preference_name, const char *new_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  g_return_val_if_fail (new_value != NULL, FALSE);
  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);

  if (pref)
    {
      nsresult rv = pref->SetCharPref (preference_name, new_value);            
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

/**
 * mozilla_preference_set_boolean: set a boolean mozilla preference
 */
extern "C" gboolean
mozilla_preference_set_boolean (const char        *preference_name,
				gboolean        new_boolean_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  
  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);
  
  if (pref)
    {
      nsresult rv = pref->SetBoolPref (preference_name,
				       new_boolean_value ? PR_TRUE : PR_FALSE);
      
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

/**
 * mozilla_preference_set_int: set an integer mozilla preference
 */
extern "C" gboolean
mozilla_preference_set_int (const char        *preference_name,
				int        new_int_value)
{
  g_return_val_if_fail (preference_name != NULL, FALSE);
  
  nsCOMPtr<nsIPref> pref = do_CreateInstance(NS_PREF_CONTRACTID);
  
  if (pref)
    {
      nsresult rv = pref->SetIntPref (preference_name,
				       new_int_value);
      
      return NS_SUCCEEDED (rv) ? TRUE : FALSE;
    }
  
  return FALSE;
}

/**
 * mozilla_init_promptService: Initialise PromptService handler.
 */
extern "C" gboolean 
mozilla_init_promptService (void)
{
	if (!pushed_startup) return FALSE;

	nsresult rv = NS_OK;

	static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
     
	nsCOMPtr<nsIFactory> promptFactory;
	rv = NS_NewPromptServiceFactory(getter_AddRefs(promptFactory));
	if (NS_FAILED(rv)) return rv;
	rv = nsComponentManager::
		RegisterFactory(kPromptServiceCID, "Prompt Service",
				"@mozilla.org/embedcomp/prompt-service;1",
				promptFactory, PR_TRUE);  
	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

/**
 * mozilla_init_ContentHandler: Initialise Content Handler.
 */
extern "C" gboolean 
mozilla_init_ContentHandler (void)
{
	if (!pushed_startup) return FALSE;

	nsresult rv = NS_OK;

	static NS_DEFINE_CID(kContentHandlerCID, NS_IHELPERAPPLAUNCHERDIALOG_IID);
     
	nsCOMPtr<nsIFactory> ContentHandlerFactory;
	rv = NS_NewContentHandlerFactory (getter_AddRefs(ContentHandlerFactory));
	if (NS_FAILED(rv)) return rv;
	rv = nsComponentManager::
		RegisterFactory(kContentHandlerCID,
				NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
				NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
				ContentHandlerFactory, PR_TRUE);  
	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

/**
 * mozilla_register_contentOverrideHandler: Registers an explicit contentHandler
 * @mime-type: The mime-type to register a contentHandler for.
 */
extern "C" gboolean mozilla_register_contentOverrideHandler (const char *mime_type)
{
        if (!pushed_startup) return FALSE;

        nsresult rv = NS_OK;
  
        static NS_DEFINE_CID(kContentOverrideHandlerCID,
        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
     
        nsCOMPtr<nsIFactory> CHFactory;
        rv = NS_NewContentOverrideHandlerFactory(getter_AddRefs(CHFactory));
        if (NS_FAILED(rv)) return FALSE;
      
        gchar *handlerName = g_strdup_printf ("Content Handler for %s",
                                              mime_type);
        gchar *contractID = g_strdup_printf (NS_CONTENT_HANDLER_CONTRACTID_PREFIX
                                             "%s",mime_type);
   
        rv = nsComponentManager::RegisterFactory(kContentOverrideHandlerCID,
                                                 handlerName,   
                                                 contractID,
                                                 CHFactory,
                                                 PR_TRUE);
        g_free (handlerName);
        g_free (contractID);

        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

/**
 * mozilla_register_stored_contentOverrideHandler: Registers contentHandlers
 * for all stored mime-types with the override action type.
 */
extern "C" gboolean mozilla_register_stored_contentOverrideHandlers (void)
{  

        gboolean rv = TRUE;
        for (GList *mime_type = g_list_first(mime_types) ;
             mime_type!=NULL ;
             mime_type = g_list_next(mime_type))
        {
                if (((MimeItem *)mime_type->data)->action == 
               		MIME_OVERRIDE_MOZILLA)
                {
                        if (!mozilla_register_contentOverrideHandler
                                (((MimeItem *)mime_type->data)->name))   
                        {
                                rv = FALSE;
                        }
                }
        }
        return rv;
}

/**
 * mozilla_unregister_stored_contentOverrideHandler: unregisters contentHandlers
 * for all stored mime-types
 */
extern "C" gboolean mozilla_unregister_stored_contentOverrideHandlers (void)
{
        if (!pushed_startup) return FALSE;
        
        nsresult rv = NS_OK;
        static NS_DEFINE_CID(kContentOverrideHandlerCID,
        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
                                                 
        nsCOMPtr<nsIFactory> CHFactory;
        
        rv = nsComponentManager::FindFactory (kContentOverrideHandlerCID,
                                              getter_AddRefs(CHFactory));
        if (NS_FAILED(rv)) return FALSE;
 
        rv = nsComponentManager::UnregisterFactory (kContentOverrideHandlerCID,
                                                    CHFactory);
 
        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

/**
 * mozilla_init_FilePicker: Register FilePicker Factory.
 */
extern "C" gboolean 
mozilla_init_FilePicker (void)
{
	if (!pushed_startup) return FALSE;

	nsresult rv = NS_OK;

	static NS_DEFINE_CID(kFilePickerCID, NS_IFILEPICKER_IID);
     
	nsCOMPtr<nsIFactory> FilePickerFactory;
	rv = NS_NewFilePickerFactory (getter_AddRefs(FilePickerFactory));
	if (NS_FAILED(rv)) return rv;
	rv = nsComponentManager::RegisterFactory(kFilePickerCID,
						 G_FILEPICKER_CLASSNAME,
						 G_FILEPICKER_CONTRACTID,
						 FilePickerFactory, PR_TRUE);  
	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
}

/**
 * mozilla_wrapper_init: initialize the mozilla wrapper
 * @attach_listener: attach event listener. Currently used for
 * links drag and drop only.
 */
extern "C" gpointer 
mozilla_wrapper_init (GaleonEmbed *embed, gboolean attach_listener)
{
	GaleonWrapper *wrapper = new GaleonWrapper ();
	wrapper->attachListener = attach_listener;
	wrapper->Init (embed);
	return wrapper;
}

/**
 * mozilla_wrapper_destroy: destroy the mozilla wrapper
 */
extern "C" gboolean
mozilla_wrapper_destroy (GaleonEmbed *embed)
{
	nsresult result;
	GaleonWrapper *wrapper;

	wrapper = (GaleonWrapper *)(embed->wrapper);
	result = wrapper->Destroy();
	embed->wrapper = NULL;
	delete wrapper;
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

extern "C" gboolean mozilla_find (GaleonEmbed *embed, 
				  WrapperSearchProperties *properties)
{
	GaleonWrapper *wrapper;
	PRUnichar *search_string;		
	PRBool didFind;

	g_return_val_if_fail (embed != NULL, FALSE);
	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	wrapper = (GaleonWrapper *)embed->wrapper;
	search_string = mozilla_locale_to_unicode (properties->search_string);
	wrapper->Find (search_string, properties->match_case, 
		       properties->backwards, &didFind);
	g_free (search_string);

	return didFind;
}

extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url, 
					const char *file_name)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	result = wrapper->SaveURI (url, file_name);

	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

extern "C" gboolean mozilla_save_link (GaleonEmbed *embed, const char *url)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	result = wrapper->SaveLink (url);

	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

extern "C" gboolean mozilla_reload (GaleonEmbed *embed)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	nsresult result = wrapper->ReloadDocument ();
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_set_zoom: Sets the zoom factor of a embed
 */
extern "C" gboolean
mozilla_set_zoom (GaleonEmbed *embed, float f)
{
	GaleonWrapper *wrapper;
	gfloat current_zoom;

	g_return_val_if_fail (embed != NULL, FALSE);
	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
	wrapper = (GaleonWrapper *)embed->wrapper;

	nsresult result = wrapper->GetZoom (&current_zoom);
	if (NS_FAILED (result) || current_zoom == f)
		return FALSE;

	result = wrapper->SetZoom (f);
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_get_zoom: Gets the zoom factor of a embed
 */
extern "C" gboolean
mozilla_get_zoom (GaleonEmbed *embed, float *f)
{
	GaleonWrapper *wrapper;

	g_return_val_if_fail (embed != NULL, FALSE);
	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
	wrapper = (GaleonWrapper *)embed->wrapper;

	nsresult result = wrapper->GetZoom (f);
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_print: print a document
 */
extern "C" gboolean
mozilla_print (GaleonEmbed *embed, PrintInfo *info)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)(embed->wrapper);

	NS_WITH_SERVICE(nsIPrintOptions, options, kPrintOptionsCID, &result);
	if (NS_FAILED(result)) return NS_ERROR_FAILURE;

	if (info->pages)
	{
		options->SetStartPageRange (info->from_page);
		options->SetEndPageRange (info->to_page);
	}

	options->SetMarginTop (info->top_margin);
	options->SetMarginBottom (info->bottom_margin);
	options->SetMarginLeft (info->left_margin);
	options->SetMarginRight (info->right_margin);

	nsCString printerString(info->printer);
	nsCString fileString(info->file);
	options->SetToFileName (fileString.ToNewUnicode());
	options->SetPrintCommand (printerString.ToNewUnicode());
	options->SetPrintToFile (info->print_to_file);

	options->SetPrintReversed (info->reversed);
	options->SetPaperSize (info->paper);

	options->SetPrintSilent (PR_TRUE);

	result = wrapper->Print(options);
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_session_history: Gets the session history and current position of a 
 * GtkMozWrapper. On return, *titles will be an array of char * wich must be freed.
 */
extern "C" gboolean
mozilla_session_history (GaleonEmbed *embed, char **titles[], 
			 int *count, int *index)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	wrapper->GetSHInfo (count, index);

	char **t = g_new(char *, *count);

	for (PRInt32 i = 0; i < *count; i++) {

		nsresult result;
		PRUnichar *unicodeTitle;

		result = wrapper->GetSHTitleAtIndex(i, &unicodeTitle);

		if (NS_FAILED(result))
		{
			return NS_OK;
		}
		
		t[i] = mozilla_unicode_to_locale (unicodeTitle);
	}
	*titles = t;

	return TRUE;
}

/**
 * mozilla_session_history_go: Goes to the SHEntry at the given index
 */
extern "C" gboolean
mozilla_session_history_go (GaleonEmbed *embed, int index)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	result = wrapper->GoToHistoryIndex (index);
	
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_get_main_document_url: Get URL of main document
 * embed: Parent widget of document
 */
extern "C" gchar * 
mozilla_get_main_document_url (GaleonEmbed *embed)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	gchar *url = NULL;
	nsresult result;

	g_return_val_if_fail (embed->wrapper != NULL, NULL); 

	result = wrapper->GetMainDocumentUrl (&url);

	return NS_SUCCEEDED (result) ? url : NULL;
}

/**
 * mozilla_get_document_url: Get URL of focused document (frame)
 * embed: Parent widget of document
 */
extern "C" gchar * 
mozilla_get_document_url (GaleonEmbed *embed)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	gchar *url = NULL;
	nsresult result;

	g_return_val_if_fail (embed->wrapper != NULL, NULL); 

	result = wrapper->GetDocumentUrl (&url);

	return NS_SUCCEEDED (result) ? url : NULL;
}

#if MOZILLA_VERSION > VERSION2(0,9)
/**
 * mozilla_get_document_title: get the document title in a locale specific
 * NULL terminated C string, using the private UniCode interface.
 */
extern "C" gchar * 
mozilla_get_document_title (GaleonEmbed *embed)
{
	PRUnichar *unicode_title;
	gchar *title;

	/* get the title in unicode */
	unicode_title = gtk_moz_embed_get_title_unichar 
		(GTK_MOZ_EMBED(embed->mozEmbed));

	/* attempt conversion */
	title = mozilla_unicode_to_locale (unicode_title);

	/* free unicode version */
	g_free (unicode_title);

	/* return it */
	return title;
}
#endif

/**
 * mozilla_get_key_event_info:
 */
extern "C" gboolean
mozilla_get_key_event_info (GaleonEmbed *embed, gpointer event, WrapperKeyEventInfo *info)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE);
	g_return_val_if_fail (event != NULL, FALSE);

	result = wrapper->GetKeyEventContext ((nsIDOMKeyEvent *)event, info);

	return NS_SUCCEEDED(result) ? TRUE : FALSE;
}

/**
 * 
 */
extern "C" gboolean
mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event, 
			      WrapperMouseEventInfo *context)
{
	nsresult result;
	return_val_if_not_embed (embed, FALSE);
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	g_return_val_if_fail (wrapper, FALSE);
	g_return_val_if_fail (event, FALSE);
	
	result = wrapper->GetMouseEventContext ((nsIDOMMouseEvent*)event, 
						context);

	/* for some reason we get different numbers on PPC, this fixes
	 * that up... -- MattA */
	if (context->button == 65536)
	{
		context->button = 1;
	}
	else if (context->button == 131072)
	{
		context->button = 2;
	}

	return NS_SUCCEEDED(result) ? TRUE : FALSE;
}


/**
 * mozilla_free_context_info_sub: free info contained in WrapperContextInfo 
 * struct.
 */
extern "C" void
mozilla_free_context_info_sub (WrapperContextInfo *ctx)
{
	g_free(ctx->linktext);

	/* created by mozilla: */
	if (ctx->link) nsMemory::Free (ctx->link);
	if (ctx->img) nsMemory::Free (ctx->img);
}

/**
 * mozilla_list_cookies: get a list of all saved cookies
 */
extern "C" GList *
mozilla_list_cookies (void)
{
	GList *cookies = NULL;
	nsresult result;

	nsCOMPtr<nsICookieManager> 
			cookieManager = do_CreateInstance (COOKIEMANAGER_ID);
	nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
	result = 
	    cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);	
	PRBool enumResult;
	for (cookieEnumerator->HasMoreElements(&enumResult) ;
	     enumResult == PR_TRUE ;
	     cookieEnumerator->HasMoreElements(&enumResult))
	{
		Cookie *c;
	
		nsCOMPtr<nsICookie> nsCookie;
		result = cookieEnumerator->GetNext (getter_AddRefs(nsCookie));
		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);

		c = g_new0 (Cookie, 1);

		nsCookie->GetHost (&(c->base.domain));
		nsCookie->GetName (&(c->name));
		nsCookie->GetValue (&(c->value));
		nsCookie->GetPath (&(c->path));
		
		PRBool isSecure;
		nsCookie->GetIsSecure (&isSecure);
		if (isSecure == PR_TRUE) 
			c->secure = g_strdup_printf ("Yes");
		else 
			c->secure = g_strdup_printf ("No");

		PRUint64 dateTime;
		nsCookie->GetExpires (&dateTime);
		c->expire = g_strdup_printf ("%s",ctime((time_t*)&dateTime));
		
		cookies = g_list_append (cookies, c);
	}	
	return cookies;
}

/**
 * mozilla_get_permissions:
 * @type: Type of permissions to get ( cookie or image )
 */

extern "C" GList *
mozilla_get_permissions (int type)
{
	GList *permissions = NULL;
	nsresult result;
	nsCOMPtr<nsIPermissionManager> 
		permissionManager = do_CreateInstance (PERMISSIONMANAGER_ID);
	nsCOMPtr<nsISimpleEnumerator> permissionEnumerator;
	result = permissionManager->
			GetEnumerator (getter_AddRefs(permissionEnumerator));
	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);
	PRBool enumResult;
	for (permissionEnumerator->HasMoreElements(&enumResult) ;
	     enumResult == PR_TRUE ;
	     permissionEnumerator->HasMoreElements(&enumResult))
	{
		nsCOMPtr<nsIPermission> nsPermission;
		result = permissionEnumerator->
				GetNext (getter_AddRefs(nsPermission));
		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);		

		PRInt32 cType;
		nsPermission->GetType (&cType);
		if (cType == type)
		{
			CookieBase *b = g_new0 (CookieBase, 1);

			nsPermission->GetHost (&(b->domain));

			PRBool cap;
			nsPermission->GetCapability (&cap);
			if (cap == PR_TRUE) 
				b->type = g_strdup_printf ("Allowed");
			else 
				b->type = g_strdup_printf ("Blocked");

			permissions = g_list_append (permissions, b);
		}
	}

	return permissions;
};

/**
 * mozilla_block_url:
 * @imgURL: url of site to block
 */
extern "C" void
mozilla_block_url (const char *imgURL)
{
	nsresult result;
	
	nsCOMPtr<nsIImgManager> 
		imageManager = do_CreateInstance (IMAGEMANAGER_ID);
	
	result = imageManager->Block (imgURL);

	g_return_if_fail (NS_SUCCEEDED(result));
}

/**
 * mozilla_set_permission:
 * @permit: whether to block or allow
 * @type: type of permission to set ( cookie or image )
 */
 
extern "C" void
mozilla_set_permission (GaleonEmbed *embed, gboolean permit, int type)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;

	if (!wrapper) return;

	wrapper->SetSitePermission (permit ? PR_TRUE : PR_FALSE, type);
}

/**
 * mozilla_remove_cookies:
 * @gone: list of cookies which are to be removed
 * @block: whether to block the originating servers in future
 */
extern "C" gboolean 
mozilla_remove_cookies (GList *gone, gboolean block)
{
	nsresult result;
	nsCOMPtr<nsICookieManager> 
		cookieManager = do_CreateInstance (COOKIEMANAGER_ID);

	for (GList *cookies = g_list_first(gone) ; cookies!=NULL ; 
	     cookies = g_list_next(cookies))
	{
		Cookie *c = (Cookie *)cookies->data;
		result = cookieManager->Remove (c->base.domain,c->name,
					 	c->path,
						block ? PR_TRUE : PR_FALSE);
		if (NS_FAILED(result)) return FALSE;
	};
	return TRUE;
}

/**
 * mozilla_remove_permissions:
 * @gone: list of cookie or image permissions which are to be removed
 * @type: type of permissions ( cookies or images )
 */
extern "C" gboolean 
mozilla_remove_permissions (GList *gone, int type)
{
	nsresult result;
	nsCOMPtr<nsIPermissionManager> 
		permissionManager = do_CreateInstance( PERMISSIONMANAGER_ID );

	for (GList *permissions = g_list_first(gone) ; permissions!=NULL;
	     permissions = g_list_next(permissions))
	{
		CookieBase *b = (CookieBase *)permissions->data;
		result = permissionManager->Remove(b->domain,type);
		if (NS_FAILED(result)) return FALSE;
	};
	return TRUE;
}

/**
 * mozilla_copy_session_history: copy the session history to another embed
 */
extern "C" gboolean
mozilla_copy_session_history (GaleonEmbed *embed, GaleonEmbed *dest)
{
	g_return_val_if_fail (embed != NULL, FALSE);
	g_return_val_if_fail (dest != NULL, FALSE);
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	GaleonWrapper *dest_wrapper = (GaleonWrapper *)dest->wrapper;
	g_return_val_if_fail (wrapper != NULL, FALSE);
	g_return_val_if_fail (dest_wrapper != NULL, FALSE);
	nsresult result = wrapper->CopyHistoryTo (dest_wrapper);
	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}

/**
 * mozilla_get_charsets: get a list of charset supported by mozilla
 */
extern "C" void
mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
{
	nsresult rv;
	PRUint32 cscount;
	PRUint32 translated_cscount = get_translated_cscount ();

	nsCOMPtr<nsIAtom> docCharsetAtom;
	nsCOMPtr<nsICharsetConverterManager2> ccm2 =
		do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
	if (!NS_SUCCEEDED(rv)) return;

	nsCOMPtr <nsISupportsArray> cs_list;
	rv = ccm2->GetDecoderList(getter_AddRefs(cs_list));
	if (!NS_SUCCEEDED(rv)) return;

	rv = cs_list->Count(&cscount);
	*charsets = g_hash_table_new(g_str_hash,g_str_equal);
	for (PRUint32 i = 0; i < cscount; i++)
	{
		nsCOMPtr<nsISupports> cssupports =
					(dont_AddRef)(cs_list->ElementAt(i));
		nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
		nsString charset_ns = nsnull, charset_title_ns = nsnull;

		/* charset name */
		rv = csatom->ToString(charset_ns);
		char *charset_str = convert_ns_string_to_c_string (charset_ns);

		/* charset readable title */
		rv = ccm2->GetCharsetTitle2(csatom, &charset_title_ns);
		char *charset_title_str = convert_ns_string_to_c_string 
						(charset_title_ns);

		if (strlen(charset_title_str) == 0)
		{
			g_free (charset_title_str);
			charset_title_str = g_strdup(charset_str);
		}
		
		for (PRUint32 j = 0; j < translated_cscount; j++)
		{
			if (g_strcasecmp (
				charset_title_str, 
				charset_trans_array[j].charset_title) == 0)
			{
				g_free (charset_title_str);
				charset_title_str = (char *) 
					_(charset_trans_array[j].charset_title);
				break;
			}
		}

		/* fill the hash and the sorted list */
		g_hash_table_insert (*charsets, charset_title_str, charset_str);
		*sorted_charset_titles = g_list_insert_sorted (
						(GList*)*sorted_charset_titles,
						(gpointer)charset_title_str,
						(GCompareFunc)g_strcasecmp); 
	}
}

/**
 * mozilla_force_character_set: force the embed to use the specified
 * character set
 */
extern "C" void
mozilla_force_character_set (GaleonEmbed *embed, char *force_character_set)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	wrapper->ForceCharacterSet (force_character_set);
}

/**
 * mozilla_clear_cache: Clear the global memory/disk cache
 */
extern "C" void
mozilla_clear_cache (unsigned long cache)
{
	nsCOMPtr<nsICacheService> 
		CacheService = do_CreateInstance (CACHESERVICE_ID);
	CacheService->EvictEntries(cache);
}

/* This nonsense is needed to get the allocators right */
static char *
convert_ns_string_to_c_string (const nsString & ns_string)
{
	char *c_string;
	char *ns_c_string = ns_string.ToNewCString ();

	if (ns_c_string == NULL)
	{
		return NULL;
	}

	c_string = g_strdup (ns_c_string);

	nsMemory::Free (ns_c_string);

	return c_string;
}

extern "C" gboolean
mozilla_get_favicon_location (GaleonEmbed *embed, gchar **url)
{
	nsresult result;
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	
	result = wrapper->GetFaviconURL (url);
	
	return NS_SUCCEEDED (result);
}

/**
 * mozilla_list_passwords: get a list of all saved username/password pairs
 */
extern "C" GList *
mozilla_list_passwords (PasswordType type)
{
	GList *passwords = NULL;
	nsresult result = NS_ERROR_FAILURE;

	nsCOMPtr<nsIPasswordManager> 
		passwordManager = do_CreateInstance (PASSWORDMANAGER_ID);
	nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
	if (type == PASSWORD)
		result = passwordManager->GetEnumerator 
				(getter_AddRefs(passwordEnumerator));
	else if (type == REJECT)
		result = passwordManager->GetRejectEnumerator 
				(getter_AddRefs(passwordEnumerator));
	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);	

	PRBool enumResult;
	for (passwordEnumerator->HasMoreElements(&enumResult) ;
	     enumResult == PR_TRUE ;
	     passwordEnumerator->HasMoreElements(&enumResult))
	{
		nsCOMPtr<nsIPassword> nsPassword;
		result = passwordEnumerator->GetNext 
					(getter_AddRefs(nsPassword));
		g_return_val_if_fail (NS_SUCCEEDED(result), NULL);

		Password *p = g_new0 (Password, 1);

		nsPassword->GetHost (&(p->host));

		if (type==PASSWORD)
		{
			PRUnichar *unicodeName;
			nsPassword->GetUser (&unicodeName);
			p->username = mozilla_unicode_to_locale (unicodeName);
		}
		
		passwords = g_list_append (passwords, p);
	}	
	return passwords;
}

/**
 * mozilla_remove_passwords:
 * @gone: list of passwords/rejcts which are to be removed
 * @type: whether list is passwords or rejects
 */
extern "C" gboolean 
mozilla_remove_passwords (GList *gone, PasswordType type)
{
	nsresult result = NS_ERROR_FAILURE;
	nsCOMPtr<nsIPasswordManager> 
		passwordManager = do_CreateInstance (PASSWORDMANAGER_ID);

	for (GList *passwords = g_list_first(gone) ; passwords!=NULL ; 
	     passwords = g_list_next(passwords))
	{
		Password *p = (Password *)passwords->data;
		if (type == PASSWORD)
		{
			PRUnichar *unicodeName = mozilla_locale_to_unicode
							(p->username);
			result = passwordManager->RemoveUser (p->host,
							      unicodeName);
			g_free (unicodeName);
		}
		else if (type == REJECT)
		{
			result = passwordManager->RemoveReject (p->host);
		};

		if (NS_FAILED(result)) return FALSE;
	};
	return TRUE;
}

/**
 * mozilla_unicode_to_utf8: Encodes unicode string to UTF-8
 * @uniStr: The unicode string to encode
 */
extern "C" gchar *
mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
{
	PRInt32 sSize,dSize;
	nsresult result;

#if MOZILLA_VERSION > VERSION2(0,9)
	const nsString str (uniStr);
#else
	const nsLocalString str = nsLiteralString (uniStr);
#endif
	sSize = str.Length ();

	nsCOMPtr<nsIUnicodeEncoder> 
		unicodeEncoder = do_CreateInstance (UTF8_ENCODER_ID);

	/* GetMaxLength returns a worst case prediction for the size
	   of the returned char*. Using it ensures that Convert will
	   not run out of space */
	result = unicodeEncoder->GetMaxLength (uniStr, sSize, &dSize);
	gchar *utfStr = g_new0 (gchar, dSize + 1);

	/* Convert must be passed the size of unicode string and
	   the size of the char* buffer so that bad things(tm)
	   won't happen. No null termination here. */
	result = unicodeEncoder->Convert (uniStr, &sSize, utfStr, &dSize);
					  
	/* Finish ensures that the encoder is left in a clean state
	   for it's next use. */
	result = unicodeEncoder->Finish (utfStr, &dSize);

	/* Normally a null would need to appended at this point but
	   cStr was initialised to zero, so there's no need */
	return utfStr;
}

/**
 * mozilla_unicode_to_locale: Encodes unicode string to something
 * valid for the current locale (which can then be used in GTK+ labels).
 * @uniStr: The unicode string to encode
 */
extern "C" gchar *
mozilla_unicode_to_locale (const PRUnichar *uniStr)
{
	PRInt32 sSize;
	wchar_t *wide;
	gchar *output;
	gint i, count;

	/* sanity */
	if (uniStr == NULL)
	{
		return NULL;
	}

#if MOZILLA_VERSION > VERSION2(0,9)
	const nsString str (uniStr);
#else
	const nsLocalString str = nsLiteralString (uniStr);
#endif
	sSize = str.Length ();

	/* allocate a wide string big enough to hold the unicode string,
	 * this is necessary since wchar_t is 32-bits with glibc */
	wide = g_new0 (wchar_t, sSize + 1);
	for (i = 0; i < sSize + 1; i++)
	{
		wide[i] = uniStr[i];
	}

	/* use glibc function to determine the size of this string once
	 * encoded to a locale specfic multibyte string */
	count = wcstombs (NULL, wide, 0);

	/* check for success */
	if (count == -1)
	{
		/* not sure what we should do here, but it seems safe to
		 * fallback to encode_to_utf8 */
		g_free (wide);
		return mozilla_unicode_to_utf8 (uniStr);
	}

	/* allocate a string big enough and do the actual conversion */
	output = g_new0 (gchar, count + 1);
	count = wcstombs (output, wide, count + 1);
	g_assert (count != -1);

	/* free wide version and return */
	g_free (wide);
	return output;
}

/**
 * mozilla_utf8_to_unicode: Decodes UTF-8 string to unicode
 * @utfStr: The unicode string to encode
 */
extern "C" PRUnichar *
mozilla_utf8_to_unicode (const gchar *utfStr)
{
	PRInt32 sSize,dSize;
	nsresult result;
	
	for (sSize = 0; utfStr[sSize] != 0; sSize++);

	nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder = 
					do_CreateInstance (UTF8_DECODER_ID);

	/* Unicode decoding is much the same as encoding as
	   described in mozilla_session_history comments, 
	   but there is no Finish function needed to complete
	   the process */
	result = unicodeDecoder->GetMaxLength (utfStr, sSize, &dSize);
	PRUnichar *uniStr = g_new0 (PRUnichar, dSize + 1);

	result = unicodeDecoder->Convert (utfStr, &sSize, uniStr, &dSize);

	return uniStr;
}

/**
 * mozilla_locale_to_unicode: Decodes a string encoded for the current
 * locale into unicode. Used for getting text entered in a GTK+ entry
 * into a form which mozilla can use.
 * @locStr: The unicode string to encode
 */
extern "C" PRUnichar *
mozilla_locale_to_unicode (const gchar *locStr)
{
	PRUnichar *uniStr;
	wchar_t *wide;
	gint i, count;

	/* sanity */
	if (locStr == NULL)
	{
		return NULL;
	}

	/* count the number of wide characters which will be produced */
	count = mbstowcs (NULL, locStr, 0);
	if (count == -1)
	{
		/* hmm, shouldnt happen but fallback to utf8 */
		g_warning ("unusual locale string: [%s]\n", locStr);
		return mozilla_utf8_to_unicode (locStr);
	}

	/* allocate and decode */
	wide = g_new0 (wchar_t, count + 1);
	mbstowcs (wide, locStr, count + 1);

	/* make a unicode string and copy into it */
	uniStr = g_new0 (PRUnichar, count + 1);
	for (i = 0; i < count + 1; i++)
	{
		uniStr[i] = wide[i];
	}

	/* free wide string and return the unicode one */
	g_free (wide);
	return uniStr;
}

/**
 * mozilla_find_gtk_parent: retrives gtkwidget pointer to parent
 * of domWindow. It will be the gtkmozembed that holds it.
 * @domWindow: nsIDOMWindow to find gtk parent of.
 */
extern "C" GtkWidget *
mozilla_find_gtk_parent (nsIDOMWindow *domWindow)
{
	nsresult result;
	
        nsCOMPtr<nsIWindowWatcher> wwatch 
		(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
        if (!wwatch) return nsnull;

        nsCOMPtr<nsIWebBrowserChrome> windowChrome;
        result = wwatch->GetChromeForWindow (domWindow,
        				     getter_AddRefs(windowChrome));
        if (NS_FAILED(result)) return nsnull;

        nsCOMPtr<nsIEmbeddingSiteWindow> window
        		(do_QueryInterface(windowChrome, &result));
        if (NS_FAILED(result)) return nsnull;

        GtkWidget *parentWindow;
        result = window->GetSiteWindow ((void **)&parentWindow);
        if (NS_FAILED(result)) return nsnull;

        return parentWindow;
}

extern "C" gboolean
mozilla_set_offline (GaleonEmbed *embed, gboolean offline)
{
	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
	nsresult result;

	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 

	result = wrapper->SetOffline (offline);

	return NS_SUCCEEDED (result) ? TRUE : FALSE;
}
